/*Style for tabs*/

.ssb-tabs.navigation-item.workspace-tab-label{
    margin: 0 1rem;
    font-size: 1.2rem;

}

.workspace-tab{
    padding: 1rem;
}

.ssb-title.workspace-tab-title{
    margin-top: 1rem;
}
