/*Style for workspace sections for dataset and variables*/

.workspace-page-wrapper{
    display: flex;
    flex-wrap: wrap;
    flex-direction: row;
    padding: 0;
}

.workspace-header{
    width: 100%;
    display:flex;
    flex-wrap: wrap;
    justify-content: space-between;
    margin-bottom: 1rem;
}

.ssb-title.workspace-title{
    margin-top: 1rem;
    width: 100%;
}

.workspace-content{
    width: 100%;
    display: flex;
    flex-direction: column;
    gap: 1rem;
}

.workspace-info-paragraph {
    display: inline-block;
    align-self: center;
    margin-bottom: 0;
}

.ssb-accordion.variable-accordion > .accordion-body {
    padding: 1rem;
}

.edit-section{
    display: flex;
    flex-direction: row;
    gap: 2rem;
    width: 100%;
    padding: 1rem 0;
}

.edit-section-form{
    display: flex;
    flex-wrap: wrap;
    gap: 2rem;
    padding: 1rem 0;
    align-items: flex-start;
}

.ssb-title.edit-section-title{
    border-bottom: 2px solid #62919A;
    margin-bottom: 1rem;
    padding-bottom: 1rem;
}

.dataset-machine-section, .variables-machine-section{
    flex-direction: column;
}

#variables-metadata-input-left-editable,
#variables-metadata-input-right-editable,
#dataset-metadata-input-left,
#dataset-metadata-input-right{
    max-width: 50%;
    width: 100%;
}

#dataset-metadata-input-left, #dataset-metadata-input-right{
    gap: 2rem;
}

#dataset-metadata-input-left,
#pseudo-metadata-input-left-pseudo
#variables-metadata-input-left-editable{
    padding-top: 0.5rem;
}

.variable-machine-section >.edit-section.dataset-machine-section{
    max-width: unset;
    width: 100%;
    justify-content: center;
}

#dataset-metadata-input-Maskingenerert, #pseudo-metadata-input-left-pseudo, #variables-metadata-input-left-machine{
    justify-content: space-between;
}

#dataset-metadata-input-Maskingenerert > div > span > span > button,
#dataset-metadata-input-right > div > span > span > button,
#dataset-metadata-input-left > div > span > span > button
{
    background-image: none;
}

.add-pseudo-button {
    float: right;
    margin-top: 0.5rem;
    margin-bottom: 1rem;
}
