/*Style for multidropdown fields*/

.input-group-container {
  display: flex;
  gap: 1rem;
  align-items: flex-start;
  padding: 0;
  margin: 0
}

.add-use-restriction {
  margin-top: auto;
}

.multidropdown-fieldset{
    border: 1px solid #274247;
    outline: #274247;
    padding: 1rem;
    width: 100%;
    position: relative;
    margin-bottom: 2rem;
}

.legend-glossary{
    background-color: white;
    padding-top: 0;
    margin: 0.3rem;
}

.multidropdown-legend{
    width: fit-content;
    font-size: 1rem;
}

.multidropdown-fieldset > .ssb-glossary.legend-glossary > .glossary-button{
    padding: 0 0.3rem;
    background-image: none;
}

.multidropdown-fieldset:focus{
    border: 1px solid #00824d;
    outline: 1px solid #00824d;
    outline-offset: 0;
}

.multidropdown-fieldset:hover{
    border: 1px solid #00824d;
    outline: 1px solid #00824d;
    outline-offset: 0;
}

.ssb-glossary.legend-glossary{
    position: absolute;
    z-index: 10;
    top: -20px;
}

.ssb-input.multidropdown-input-component{
    max-width: unset;
    width: 100%;
}

.multidropdown-fieldset > .ssb-glossary.legend-glossary > .glossary-popup {
    margin-left: 20px;
    left: unset;
    top: unset;
    bottom: 5px;
}

.multidropdown-fieldset > .ssb-glossary.legend-glossary > .glossary-popup::after {
    left: -10px;
    margin-left: -10px;
    top: unset;
    bottom: 5px;
    transform: rotateX(-90deg);
    -webkit-transform: rotate(-90deg);
}

.ssb-glossary.legend-glossary > .glossary-popup .content-box > .glossary-closing{
    margin-top: 1.8rem;
}

.input-group-container {
  display: flex;
  flex-direction: column;
  gap: 1rem;
}

.input-group-row {
  display: flex;
  gap: 1rem;
  align-items: flex-start;
  padding-bottom: 0.8rem;
}

.ssb-input.input-component:has(input[id*="date-use_restrictions"]) {
  max-width: 200px;
  flex: none;
}

.multidropdown-delete-button {
  position: relative;
  width: 32px;
  height: 32px;
  border-radius: 50%;
  border: 1px solid #d32f2f;
  background-color: #f44336;
  color: white;
  cursor: pointer;
  display: flex;
  align-items: center;
  justify-content: center;
  font-size: 20px;
  font-weight: bold;
  transition: background-color 0.2s ease, transform 0.1s ease;
  margin-top: 30px;
  flex-shrink: 0;
}

.multidropdown-delete-button::before {
  content: "";
  position: absolute;
  width: 14px;
  height: 2px;
  background-color: white;
}

.multidropdown-delete-button:hover {
  background-color: #d32f2f;
  transform: scale(1.05);
}

.multidropdown-delete-button:active {
  transform: scale(0.95);
}

.multidropdown-delete-button:focus {
  outline: 2px solid #d32f2f;
  outline-offset: 2px;
}

[id^="label_date-use_restrictions-"][id*="-dataset-metadata-multidropdown-input"] {
  margin-bottom: 0;
}

.multidropdown-fieldset > button.ssb-btn.secondary-btn {
  margin-top: 0.2rem;
}
