SECTION_WRAPPER_ID = "section-wrapper-id"

VARIABLES_INFORMATION_ID = "variables-information"
ACCORDION_WRAPPER_ID = "accordion-wrapper"
ADD_USE_RESTRICTION_BUTTON = "add-use-restriction-button"
FORCE_RERENDER_COUNTER = "force-rerender-counter"
USE_RESTRICTION_STORE = "use-restriction-store"
USE_RESTRICTION_OPTION_STORE = "use-restriction-options-store"
USE_RESTRICTION_ID_STORE = "use-restriction-id-store"
USE_RESTRICTION_LIST_CONTAINER = "use-restriction-list-container"
