/*Style for input components and checkbox components*/

.ssb-input.input-component > span > .ssb-glossary.info-glossary > .glossary-popup {
    margin-left: 20px;
    left: unset;
    top: unset;
    bottom: 5px;
}

.ssb-input.input-component > span > .ssb-glossary.info-glossary > .glossary-popup::after {
    left: -10px;
    margin-left: -10px;
    top: unset;
    bottom: 5px;
    transform: rotateX(-90deg);
    -webkit-transform: rotate(-90deg);
}

.ssb-input.input-component input[readonly]{
    border: 1px solid #274247 !important;
    outline: unset !important;
    outline-offset: unset !important;
}

.ssb-input.input-component input[readonly]:focus{
    border: 1px solid #274247 !important;
    outline: unset !important;
    outline-offset: unset !important;
}

.ssb-input.input-component input[readonly]:hover{
    border: 1px solid #274247 !important;
    outline: unset !important;
    outline-offset: unset !important;
}

.ssb-input.input-component{
    max-width: unset;
    margin: 0;
}

.ssb-input-label{
    margin-bottom: 0;
}
div > .ssb-input.input-component{
    margin: 0;
}

#dataset-metadata-input-Maskingenerert > .ssb-input.input-component,
#pseudo-metadata-input-left-pseudo > .ssb-input.input-component,
#variables-metadata-input-left-machine > .ssb-input.input-component{
    max-width: 720px;
    width: 100%;
}

#pseudo-metadata-input-left-pseudo > .ssb-input.input-component,
#variables-metadata-input-left-machine > .ssb-input.input-component{
    padding: 0;
}

.metadata-checkbox-field{
    width: 100%;
}

.form-check.ssb-checkbox{
    align-self: center;
}

.form-check{
    padding-left: 0;
}

label.form-check-label{
    padding-left: 1rem;
    font-size: 1.2rem;
}

#input_version-dataset-metadata-input{
    margin-right: 1rem;
}


.ssb-glossary.info-glossary{
    max-width: fit-content;
}

.ssb-glossary.datadoc-glossary > .glossary-popup .content-box > .glossary-closing{
    margin-top: 1.8rem;
}

#dataset-metadata-input-Maskingenerert > .ssb-input .input-component > .ssb-glossary > .glossary-button,
#variables-metadata-input-left-machine > div > span > span > button,
#pseudo-metadata-input-left-pseudo > div > span > span > button {
    background-image: none;
}


@media only screen and (max-width: 1900px) {
    #variables-metadata-input-left-machine > .ssb-input.input-component,
    #pseudo-metadata-input-left-pseudo > .ssb-input.input-component,
    #dataset-metadata-input-Maskingenerert > .ssb-input.input-component {
        max-width: 46%;
    }
}
